package br.com.stoneage.identify.identify_sdk_flutter.plugins

import android.content.Context
import android.content.Intent
import br.com.stoneage.identify.enums.CameraCaptureModeEnum
import br.com.stoneage.identify.identify_sdk_flutter.enums.DocumentCaptureModeEnum
import br.com.stoneage.identify.identify_sdk_flutter.enums.DocumentSideEnum
import br.com.stoneage.identify.identify_sdk_flutter.errors.IdentifyError
import br.com.stoneage.identify.identify_sdk_flutter.errors.UnknownError
import br.com.stoneage.identify.identify_sdk_flutter.models.CollectDocumentResult
import br.com.stoneage.identify.identify_sdk_flutter.models.DocumentOptionalValidations
import br.com.stoneage.identify.identify_sdk_flutter.models.common.IdentifySdkPluginActivityResult
import br.com.stoneage.identify.identify_sdk_flutter.models.plugins.IIdentifySdkPluginDocument
import br.com.stoneage.identify.identify_sdk_flutter.utils.IntentUtils.getParcelable
import br.com.stoneage.identify.models.ValidateDocumentParameters
import br.com.stoneage.identify.sdk.DocumentBackActivity
import br.com.stoneage.identify.sdk.DocumentFrontActivity
import br.com.stoneage.identify.sdk.STACameraConfig
import br.com.stoneage.identify.models.DocumentActivityErrorResult as NativeErrorResult
import br.com.stoneage.identify.models.DocumentActivityResult as NativeResult

class IdentifySdkPluginDocument(private val context: Context) : IIdentifySdkPluginDocument,
    IdentifySdkPluginActivityResult<CollectDocumentResult>() {
    override fun collect(
        documentSide: DocumentSideEnum,
        captureMode: DocumentCaptureModeEnum?,
        optionalValidations: DocumentOptionalValidations?,
        callback: (Result<CollectDocumentResult>) -> Unit
    ) {
        mCallback = callback

        val intent = when (documentSide) {
            DocumentSideEnum.DOCUMENT_FRONT -> collectDocumentFront(
                captureMode,
                optionalValidations
            )

            DocumentSideEnum.DOCUMENT_BACK -> collectDocumentBack(captureMode, optionalValidations)
        }

        startActivityForResult(intent)
    }

    override fun handleResultOk(data: Intent?) {
        val result = data?.getParcelable<NativeResult>("result")

        if (result == null || result.imageFile?.exists() == false) {
            mCallback(Result.failure(UnknownError()))
            return
        }

        val resultData = CollectDocumentResult.fromNativeResult(result)
        mCallback(Result.success(resultData))
    }

    override fun handleResultCanceled(data: Intent?) {
        return try {
            val error = data?.getParcelable<NativeErrorResult>("error")

            if (error?.errorStatus == null) {
                mCallback(Result.failure(UnknownError()))
                return
            }

            mCallback(
                Result.failure(
                    IdentifyError.fromNativeResult(
                        error.errorStatus,
                        error.errorMessage,
                        error
                    )
                )
            )
        } catch (e: Exception) {
            mCallback(Result.failure(e))
        }
    }

    private fun collectDocumentFront(
        captureMode: DocumentCaptureModeEnum?,
        optionalValidations: DocumentOptionalValidations?,
    ): Intent {
        val intent = Intent(this.context, DocumentFrontActivity::class.java)
        val cameraConfig = getCameraConfig(captureMode)
        intent.putExtra(DocumentFrontActivity.BUNDLE_CAMERA_CONFIG, cameraConfig)
        val validationParameters = ValidateDocumentParameters(optionalValidations?.doOcr ?: false)
        intent.putExtra("parameters", validationParameters)
        return intent
    }

    private fun collectDocumentBack(
        captureMode: DocumentCaptureModeEnum?,
        optionalValidations: DocumentOptionalValidations?,
    ): Intent {
        val intent = Intent(this.context, DocumentBackActivity::class.java)
        val cameraConfig = getCameraConfig(captureMode)
        intent.putExtra(DocumentBackActivity.BUNDLE_CAMERA_CONFIG, cameraConfig)
        val validationParameters = ValidateDocumentParameters(optionalValidations?.doOcr ?: false)
        intent.putExtra("parameters", validationParameters)
        return intent
    }

    private fun getCameraConfig(captureMode: DocumentCaptureModeEnum?): STACameraConfig {
        val cameraCaptureMode = captureMode?.toNativeCaptureMode() ?: CameraCaptureModeEnum.MANUAL
        return STACameraConfig(cameraCaptureMode)
    }
}