import STAIdentify

extension STAError {
    func mapToNativeError() -> IdentifyNativeError {
        switch self {
        case .unauthorizedLocation:
            return IdentifyException(code: .permissionDenied,
                                     message: self.localizedDescription)
        case .noResponse:
            return IdentifyException(code: .connectionError,
                                     message: self.localizedDescription)
        case .invalidSessionSettings:
            return IdentifyException(code: .invalidSession,
                                     message: self.localizedDescription)
        case .httpRequestError(let error):
            return IdentifyException(code: .connectionError,
                                     message: error.localizedDescription)
        case .httpClientError(let statusCode, let data),
                .httpServerError(let statusCode, let data):
            return IdentifyException(code: .apiError,
                                     message: "Parsing error: Status: \(statusCode) - Data: \(data)")
        case .undecodableResponseError(let data):
            return IdentifyException(code: .apiError,
                                     message: "Parsing error: \(data)")
        case .unauthorizedCamera:
            return IdentifyException(code: .cameraUnavailable,
                                     message: self.localizedDescription)
        case .canceledByUser:
            return IdentifyException(code: .canceledByUser,
                                     message: self.localizedDescription)
            
        case .invalidParameter(parameterName: let parameterName, description: let description):
            if parameterName == "biometricValidation" {
                return IdentifyException(code: .invalidCpf,
                                         message: self.localizedDescription)
            }

            return IdentifyException(code: .unknownError,
                                     message: "InvalidParameter: \(parameterName) - Description: \(description)")
        case .collectDocumentValidationError(let documentValidationError):
            let validationErrors = documentValidationError.validationErrors.map { $0.toDocumentValidationErrorEnum() }
            
            return CollectDocumentException(message: self.localizedDescription,
                                            validationErrors: validationErrors)
        case .documentDetectionError(let error):
            let validationErrors = [ DocumentValidationErrorEnum.documentDetectionFailure ]
            
            return CollectDocumentException(message: error.localizedDescription,
                                            validationErrors: validationErrors)
        case .collectSelfieValidationError(let selfieValidationError):
            let validationErrors = selfieValidationError.validationErrors.map { $0.toSelfieValidationErrorEnum() }
            let optionalValidations = selfieValidationError.optionalValidations?.toSelfieOptionalValidations()
            
            return CollectSelfieException(message: self.localizedDescription,
                                          validationErrors: validationErrors,
                                          optionalValidations: optionalValidations)
        case .collectDocumentOcrError(let ocrError):
            return IdentifyException(code: .collectOcrError,
                                     message: ocrError.error)

        case .liveSelfieInvalidNumberOfGestures:
            return IdentifyException(code: .invalidGesturesNumber,
                                     message: self.localizedDescription)
        case .liveSelfieValidationError(_):
            // TODO: Map to specific error and create validationErrors
            return IdentifyException(code: .liveSelfieValidationError,
                                     message: self.localizedDescription)
        case .oneTimeCodeError(_):
            // TODO: Map to specific error and create validationErrors
            return IdentifyException(code: .oneTimeCodeError,
                                     message: self.localizedDescription)
        case .collectDigitalDocumentError(_):
            // TODO: Map to specific error and create validationErrors
            return IdentifyException(code: .digitalDocumentValidationError,
                                     message: self.localizedDescription)
        case .unknownError(let error):
            return IdentifyException(code: .unknownError,
                                     message: error.description)
        }
    }
}
