import 'package:flutter/services.dart';
import 'package:yaml/yaml.dart';

import 'src/enums/all.dart';
import 'src/exceptions/all.dart';
import 'src/models/all.dart';
import 'src/plugins/all.dart';

//Enums
export 'src/enums/all.dart';
//Exceptions
export 'src/exceptions/all.dart';
//Models
export 'src/models/all.dart';
// Codecs cannot be exported

/// Main class that wraps all Identify features
abstract class IdentifySdkFlutter {
  // CORE
  /// This function is used to configure the access token for the Identify SDK.
  ///
  /// _**[token]**_ - The access token returned by the
  /// [Identify API](https://identifydocs.stoneage.com.br/webapp/manuais-integracao/integracao-sdk#autentica%C3%A7%C3%A3o).
  ///
  /// _**[stage]**_ - The stage that will be used to access the Identify
  /// features.
  ///
  /// _**[theme]**_ - Theme used for customization.
  static Future<void> initializeSession(String token, String stage,
      [IdentifyTheme? theme]) async {
    await IdentifySdkPluginCore().initializeSession(token, stage, theme);
  }

  /* static Future<void> initializeSessionDev(
      String token, IdentifyStageEnum stage /*,
      [IdentifyTheme? theme]*/
      ) async {
    await IdentifySdkPluginCore().initializeSession(token, stage.name, null);
  } */

  // COLLECT DOCUMENT
  /// This function request the user to take a document's photo: front or back.
  /// The document front is the side which has the user face photo.
  /// The document back is opposite side.
  ///
  /// _**[documentSide]**_ - Side for the capture.
  ///
  /// _**[captureMode]**_ - Whether the document should be automatic or
  /// triggered manually by the user. Default value is
  /// [DocumentCaptureModeEnum.manual].
  ///
  /// _**[optionalValidations]**_ - Object with all optional validations for the
  /// request.
  ///
  ///  **Returns** [CollectDocumentResult] with the image and the document's
  ///  information.
  ///
  /// **Throws** [CollectDocumentException] if a validation error occurred.
  ///
  /// **Throws** [IdentifyException] if a known Identify error occurred.
  static Future<CollectDocumentResult> collectDocument(
      DocumentSideEnum documentSide,
      [DocumentCaptureModeEnum? captureMode = DocumentCaptureModeEnum.manual,
      DocumentOptionalValidations? optionalValidations]) async {
    return await _executeWithExceptionHandling(() async =>
        await IdentifySdkPluginDocument()
            .collect(documentSide, captureMode, optionalValidations));
  }

  //SELFIE
  /// This function starts a selfie capture flow. It requests a selfie from the user.
  ///
  /// _**[parameters]**_ - Defines optional validations to be executed on the
  /// captured selfie.
  ///
  /// **Returns** [CollectSelfieResult] with the image of the selfie from the
  /// user. If the capture fails, it will return an error.
  ///
  /// **Throws** [CollectSelfieException] if a validation error occurred.
  ///
  /// **Throws** [IdentifyException] if a known Identify error occurred.
  static Future<CollectSelfieResult> collectSelfie(
      [CollectSelfieParameters? parameters]) async {
    return await _executeWithExceptionHandling(
        () async => await IdentifySdkPluginSelfie().collect(parameters));
  }

  //OCR
  /// This function is used to return OCR data extracted from a [collectDocument] validation.
  ///
  /// _**[requestId]**_ - Request ID returned from a [collectDocument] validation,
  /// with <c>doOcr = true</c> as an optional validation. For more information,
  /// see [DocumentOptionalValidations] class.
  ///
  /// **Returns** [CollectOcrResult] with the status of the OCR processing and,
  /// if it is finished, information extracted from the document. Otherwise,
  /// it will return a message with the processing status.
  ///
  /// **Throws** [IdentifyException] if a known Identify error occurred.
  static Future<CollectOcrResult> collectOcr(String requestId) async {
    return await _executeWithExceptionHandling(
        () async => await IdentifySdkPluginOcr().collect(requestId));
  }

  // DEVICE INFORMATION
  /// This function gathers information about the user device. The geolocation
  /// information can be optionally retrieved. In that case, the app must have
  /// permission to access the GPS data and the user must allow this information
  /// to be retrieved.
  ///
  /// _**[geolocation]**_ - Whether to retrieve the geolocation information.
  ///
  /// **Returns** [DeviceInformationResult] with all the information from the
  /// user's device.
  ///
  /// **Throws** [IdentifyException] if a known Identify error occurred.
  static Future<DeviceInformationResult> getDeviceInformation(
      {required bool geolocation}) async {
    return await _executeWithExceptionHandling(() async {
      DeviceInformationResult result =
          await IdentifySdkPluginDeviceInformation()
              .getDeviceInformation(geolocation: geolocation);
      result.flutterSdkVersion = await _getSdkVersion();
      return result;
    });
  }

  static Future<T> _executeWithExceptionHandling<T>(
      Future<T> Function() function) async {
    try {
      return await function();
    } on PlatformException catch (e) {
      //NOTE: All exceptions will be PlatformExceptions. The specific exception
      // will be in the `details` property, which is dynamic. It must be checked.
      final details = e.details;

      if (details is IdentifyException) {
        throw details;
      }

      //NOTE: All other exceptions must rethrow
      rethrow;
    }
  }

  static Future<String> _getSdkVersion() async {
    final pubspec = await rootBundle
        .loadString('packages/identify_sdk_flutter/pubspec.yaml');
    final pubspecYaml = loadYaml(pubspec);
    return pubspecYaml['version'];
  }
}
