import 'android_only_device_information.dart';
import 'ios_only_device_information.dart';
import 'location_data.dart';

/// All device's information captured during the [getDeviceInformation] flow.
class DeviceInformationResult {
  /// @nodoc
  DeviceInformationResult({
    required this.vendorId,
    required this.deviceModel,
    required this.manufacturer,
    required this.systemName,
    required this.usesMetricSystem,
    required this.isRooted,
    required this.timeZone,
    required this.preferredLanguages,
    required this.carriers,
    this.flutterSdkVersion,
    this.deviceStorage,
    this.currencyCode,
    this.location,
    this.wiFiIp,
    this.publicIp,
    this.nativeSdkVersion,
    this.androidOnly,
    this.iosOnly,
  });

  /// An identifier unique per device and vendor. This field have the same value
  /// for all apps of the same vendor in a given device.
  String vendorId;

  /// The device model.
  ///
  /// * For Android it's the model described by the manufacturer. For example:
  /// "ASUS_Z01KD".
  ///
  /// * For iOS it's the Apple’s Device ID for the device. For example: "iPhone9,3".
  String deviceModel;

  /// The manufacturer of the device. For example: "Asus". In iOS devices, the
  /// manufacturer is always "Apple".
  String manufacturer;

  /// The name of the device's system.
  ///
  /// *  Android devices, it's the name of the Android version. For example:
  /// "Lollipop" or "P".
  ///
  /// * For iOS devices, it's the OS name followed by the OS version. For
  /// example: "iOS 14.0.1" or "iPadOS 14.0.1".
  ///
  /// **Note**: For iPhones, the OS name is always "iOS". For iPads running iOS
  /// 13 or higher it’s "iPadOS". For iPads running earlier versions of the OS,
  /// it's also "iOS".
  String systemName;

  /// Whether or not the device is configured to use the metric system.
  bool usesMetricSystem;

  /// Whether or not the device is rooted (or jailbroken in case of iOS devices).
  bool isRooted;

  /// A string representing the device's IANA timezone identifier. For example
  /// "America/Sao_Paulo". For a list of timezone identifiers, see
  /// [this link](https://en.wikipedia.org/wiki/List_of_tz_database_time_zones)
  String timeZone;

  /// A list of the preferred languages of the device. For example:
  /// "['pt-BR', 'en-BR']".
  List<String?> preferredLanguages;

  /// A list containing the device's phone carriers' name.
  List<String?> carriers;

  /// The version of the Identify Flutter SDK the device is running.
  String? flutterSdkVersion;

  /// The device internal storage.
  ///
  /// * For Android it's the total amount of storage of the device's Internal
  /// Memory partition.
  ///
  /// * For iOS it's the total amount of storage of the device. For example:
  /// 127,93 GB.
  ///
  /// This property is optional.
  String? deviceStorage;

  /// The code of the currency used by the device. For example: "BRL". This
  /// property is optional.
  String? currencyCode;

  /// The geolocation information. This property is optional.
  LocationData? location;

  /// If the device is connected to a WiFi network, its IP address on this
  /// network. This property is optional.
  String? wiFiIp;

  /// The device public (i.e. external) IP address. This property is optional.
  String? publicIp;

  /// The version of the Identify native (i.e. iOS/Android) SDK the device is
  /// running.
  String? nativeSdkVersion;

  /// Some device information that is available only on Android.
  AndroidOnlyDeviceInformation? androidOnly;

  /// Some device information that is available only on iOS.
  IosOnlyDeviceInformation? iosOnly;

  /// @nodoc
  Object encode() {
    return <Object?>[
      vendorId,
      deviceModel,
      manufacturer,
      systemName,
      usesMetricSystem,
      isRooted,
      timeZone,
      preferredLanguages,
      carriers,
      flutterSdkVersion,
      deviceStorage,
      currencyCode,
      location?.encode(),
      wiFiIp,
      publicIp,
      nativeSdkVersion,
      androidOnly?.encode(),
      iosOnly?.encode(),
    ];
  }

  /// @nodoc
  static DeviceInformationResult decode(Object result) {
    result as List<Object?>;
    return DeviceInformationResult(
      vendorId: result[0]! as String,
      deviceModel: result[1]! as String,
      manufacturer: result[2]! as String,
      systemName: result[3]! as String,
      usesMetricSystem: result[4]! as bool,
      isRooted: result[5]! as bool,
      timeZone: result[6]! as String,
      preferredLanguages: (result[7] as List<Object?>?)!.cast<String?>(),
      carriers: (result[8] as List<Object?>?)!.cast<String?>(),
      flutterSdkVersion: result[9] as String?,
      deviceStorage: result[10] as String?,
      currencyCode: result[11] as String?,
      location: result[12] != null
          ? LocationData.decode(result[12]! as List<Object?>)
          : null,
      wiFiIp: result[13] as String?,
      publicIp: result[14] as String?,
      nativeSdkVersion: result[15] as String?,
      androidOnly: result[16] != null
          ? AndroidOnlyDeviceInformation.decode(result[16]! as List<Object?>)
          : null,
      iosOnly: result[17] != null
          ? IosOnlyDeviceInformation.decode(result[17]! as List<Object?>)
          : null,
    );
  }
}
